#include <string>
#include <iostream>
#include "tga.hpp"
#include "tga_e.hpp"
#include "DataBuffer.hpp"


void tga::TGA::readHeader() throw()
    //throw (tga::TypeError)
{
    unsigned char byte;
    int twobytes;
    
    this->setImageIDSize(this->nextByte());
    this->setColormapType(this->nextByte());
    this->setType(this->nextByte());
    try {
	this->checkType();
    } catch (tga::TypeError& e) {
	throw;
    }
    
    twobytes = this->nextByte();
    byte = this->nextByte();
    twobytes += (static_cast<int>(byte)) << 8;
    this->setColormapIndex(twobytes);


    twobytes = this->nextByte();
    byte = this->nextByte();
    twobytes += (static_cast<int>(byte)) << 8;
    this->setColormapLength(twobytes);

    this->setColormapBits(this->nextByte());

    twobytes = this->nextByte();
    byte = this->nextByte();
    twobytes += (static_cast<int>(byte)) << 8;
    this->setXOrigin(twobytes);

    twobytes = this->nextByte();
    byte = this->nextByte();
    twobytes += (static_cast<int>(byte)) << 8;
    this->setYOrigin(twobytes);

    twobytes = this->nextByte();
    byte = this->nextByte();
    twobytes += (static_cast<int>(byte)) << 8;
    this->setWidth(twobytes);
    
    twobytes = this->nextByte();
    byte = this->nextByte();
    twobytes += (static_cast<int>(byte)) << 8;
    this->setHeight(twobytes);

    /* pixel depth in bits */
    this->setBytesPerPixel(this->nextByte() / 8);
    
    this->setImageDescriptor(this->nextByte());

    unsigned char offset;
    offset = 18 + this->getImageIDSize();
    offset += (this->getColormapBits() * this->getColormapLength());
    this->setImageDataOffset(offset);
    
    this->setHeaderValid(true);
}

void tga::TGA::checkType() throw() //throw (tga::TypeError)
{
    if (this->getType() != 2) {
	throw tga::TypeError(this->getType());
    }
}

void tga::TGA::readData(DataBuffer& data, int size)
{
    data.readFile(this->file, size);
}
